import logging
import sys

# Module parameters and constants
APP_NAME = "SmileyFace Unreal Tournament 4 Server Panel"
APP_AUTHOR = "Mathew Guest"
APP_VERSION = "0.1.0"

APP_CONFIG_FILENAME = "config.ini"

# config.spec is relative to the module src directory and is the
# config specification (structure, names, and types of config file)
APP_CONFIGSPEC_FILENAME = "config.spec"

# Check and gracefully fail if the user needs to install a 3rd-party dep.
required_lib_names = ["appdirs", "configobj", "colorlog"]


def check_env_has_dependencies(required_lib_names):
    """
    Attempts to import each module and gracefully fails if it doesn't
    exist.
    """
    rc = True
    for libname in required_lib_names:
        try:
            __import__(libname)
        except ImportError as ex:
            print("missing third-part library: ", ex, file=sys.stderr)
            rc = False
        except Exception as ex:
            print(ex, type(ex))
            rc = False
    return rc


if not check_env_has_dependencies(required_lib_names):
    print("refusing to load program without installed dependencies", file=sys.stderr)
    raise ImportError("python environment needs third-party dependencies installed")

# Exposed from sub-modules:
from .app import start_app
