import app_skellington
from app_skellington import _util

from . import (
    datalayer,
    hub_machine,
    scrape_latest,
)


class SmileyFace(app_skellington.ApplicationContainer):
    def __init__(self, *args, **kwargs):
        filename = "config.spec"
        self.configspec_filepath = _util.get_asset(__name__, filename)

        config_filepath = self._get_config_filepath("smileyface-ut4", "", "hub-config.ini")

        super().__init__(
            configspec_filepath=self.configspec_filepath,
            configini_filepath=config_filepath,
            app_name="SmileyFace UT4 Server Panel",
            app_author="Mathew Guest",
            app_version="0.1",
            *args,
            **kwargs,
        )

    def _cli_options(self):
        pass

    def _command_menu(self):
        sm_root = self.cli.init_submenu("command")
        _util.register_class_as_commands(self, sm_root, hub_machine.UT4ServerMachine)

        sm_scrape = sm_root.create_submenu("scrape")
        _util.register_class_as_commands(self, sm_scrape, scrape_latest.ScrapeUt4Pugs)

        _util.register_class_as_commands(self, sm_scrape, scrape_latest.ScrapeUtcc)

        _util.register_class_as_commands(self, sm_scrape, scrape_latest.LocalFs)

    def _services(self):
        self["model"] = lambda: hub_machine.UTServerMachine(self.ctx)
        self.dal = datalayer.DataLayer(self.ctx)
        self["dal"] = lambda: self.dal
        self["datalayer"] = lambda: datalayer.DbFuncs(self.ctx, self.dal)

        # self['localfs'] = lambda: datalayer.LocalFs(self.ctx, datalayer)

    def interactive_shell(self):
        pass

    def invoke_from_cli(self):
        rc = self.load_command()
        if not rc:
            print("Invalid command. Try -h for usage")
            return
        # load config
        self.invoke_command()

    def usage(self):
        s = """
Unreal Tournament 4 Server Build and Deploy Script

A list of commands is shown below. 

List commands and usage:
        ./ut4-server-ctl.sh 

Show help for a specific command:
        ./ut4-server-ctl.sh --help <COMMAND>

Here is the list of sub-commands with short syntax reminder:
        ./ut4-server-ctl.sh 1click-deploy
        ./ut4-server-ctl.sh clean-instance
        ./ut4-server-ctl.sh create-directories
        ./ut4-server-ctl.sh download-linux-server
        ./ut4-server-ctl.sh download-logs
        ./ut4-server-ctl.sh generate-instance
        ./ut4-server-ctl.sh start-server
        ./ut4-server-ctl.sh stop-server
        ./ut4-server-ctl.sh upload-redirects
        ./ut4-server-ctl.sh upload-server

Typical Usage:
        1) You need to either configure the remote server hostnames (both game server and remote redirect server)
        in the vars files. Edit vars with a text editor, or edit the defaults in this file, or manually type
        them interactively when prompted* (coming soon).

        e.g.:
                PROJECT_DIR="/path/to/this/script/directory/on/local/machine"
                REMOTE_GAME_HOST="54.123.456.10"
                REMOTE_GAME_DIR="/home/ut4/hub-instance"

                REMOTE_REDIRECT_HOST="45.321.654.10"
                REMOTE_REDIRECT_DIR="/srv/ut4-redirect/"

        2) You need to download the latest Linux Server release from Epic. Do
        this with the 'download-server' command.

        e.g.:
                ./ut4-server-ctl.sh download-server

        3) Add and configure custom maps, mutators, rulesets, and hub configuration to your *local* project folder.
        This is done by modifying the files in the project directory. With the current environment variables,
        this is set to be:
        
                "$PROJECT_DIR"

        This is the fun part! Connect with UTCC or UTZONE.DE (unaffiliated) to find custom content to put on
        your hub.

        4) 1click-deploy to update the remote hub and redirect with your latest content. You're done! Rinse and repeat.

        e.g.:
                ./ut4-server-ctl.sh 1click-deploy

        Alternatively, this command is equivalent to running the separate commands. If you'd like more fine-grained
        control, you can run them individually.
                ./ut4-server-ctl.sh generate-instance
                ./ut4-server-ctl.sh upload-redirects
                ./ut4-server-ctl.sh upload-server
                ./ut4-server-ctl.sh restart-server
"""
        print(s)


def start_app():
    app = SmileyFace()
    app.invoke_from_cli()
