import os
import sqlite3

import appdirs

from smileyface import myutil


class DataLayer:
    def __init__(self, ctx):
        self.ctx = ctx
        self._db_conn = None

    @property
    def db_conn(self):
        if self._db_conn is None:
            self._db_conn = self._create_db_connection()
        return self._db_conn

    def _create_db_connection(self):
        local_db_filename = self.ctx.config["app"]["sqlite_filename"]
        appdir = appdirs.user_data_dir("smileyface")
        fullpath = os.path.join(appdir, local_db_filename)
        self.ctx.log["ut4"].info("sqlite3 filename: %s", fullpath)

        myutil.ensure_dir_exists(fullpath)

        db = sqlite3.connect(fullpath)
        return db

    def commit(self):
        self.ctx.log["db"].info("commit()")
        self.db_conn.commit()
