import datetime
import os

import app_skellington._util as apputil
import appdirs
import sqlparse

from smileyface import myutil, structs


class DbFuncs:
    def __init__(self, ctx, dal):
        self.ctx = ctx
        self.dal = dal

    def create_tables(self):
        sql_filename = apputil.get_asset(__name__, "create_schema.sql")
        with open(sql_filename) as fp:
            contents_sql = fp.read()
            stmts = sqlparse.split(contents_sql)

        conn = self.dal.db_conn
        curs = conn.cursor()
        for stmt in stmts:
            print("----")
            print(stmt)
            curs.execute(stmt)

        self.dal.commit()

    def truncate_tables(self):
        conn = self.dal.db_conn
        curs = conn.cursor()
        sql = """
truncate file_paks;
        """
        for stmt in sqlparse.split(sql):
            curs.execute(stmt)
        self.dal.commit()

    def insert_filepak_record(self, record):

        # NOTE(MG) datetime parameter logic could be improved and more complete
        conn = self.dal.db_conn
        curs = conn.cursor()
        sql = """
insert into file_paks (
    file_pak_id,
    fullpath,
    filename,
    md5sum,
    record_created_at,
    record_updated_at
) values (
    ?, ?, ?, ?,
    datetime('now', 'localtime'),
    datetime('now', 'localtime')
)
on conflict(filename) do update set
    file_pak_id = excluded.file_pak_id,
    fullpath = excluded.fullpath,
    filename = excluded.filename, -- unique key
    md5sum = excluded.md5sum,
    --created at does not update
    record_updated_at = datetime('now', 'localtime')
                """
        args = (
            record.file_pak_id,
            record.fullpath,
            record.filename,
            record.md5sum,
            # record.record_created_at,
            # record.record_updated_at
        )
        # print(sql)
        curs.execute(sql, args)
        self.dal.commit()

    def mark_filepak_validated_state(self, rec_id, validate_state, src, remote_src_md5):
        # validation data src
        conn = self.dal.db_conn
        curs = conn.cursor()
        sql = """
update file_paks
set
    validated_state = ?,
    validated_at = datetime('now', 'localtime'),
    validated_against = ?,
    remote_src_md5 = ?
where
    file_pak_id = ?
    
"""
        args = (validate_state, src, remote_src_md5, rec_id)
        curs.execute(sql, args)
        conn.commit()

    def query_filepak(self, filename):
        conn = self.dal.db_conn
        curs = conn.cursor()
        sql = """
select 
    file_pak_id, fullpath, filename,
    md5sum, record_created_at, record_updated_at
from
    file_paks
where
    lower(filename) = lower(?)"""
        args = (filename,)
        # print(sql)
        curs.execute(sql, args)
        rows = curs.fetchall()
        output = []
        for r in rows:
            filepak = structs.FilePak()
            print(r)
            filepak.file_pak_id = r[0]
            filepak.fullpath = r[1]
            filepak.filename = r[2]
            filepak.md5sum = r[3]
            filepak.record_created_at = r[4]
            filepak.record_updated_at = r[5]
            output.append(filepak)
        if len(output) == 0:
            return
        elif len(output) == 1:
            return output[0]
        elif len(output) > 1:
            input("<breakpoint> unexpected two rows returned from db when expecting to be unique")
            return output
        return output

    def query_invalid_filepaks(self):
        conn = self.dal.db_conn
        curs = conn.cursor()
        sql = """
select 
    file_pak_id, fullpath, filename,
    md5sum, 
    validated_state, validated_at, validated_against,
    record_created_at, record_updated_at
from
    file_paks
where
    lower(filename) = lower(?)"""
        args = (filename,)
        # print(sql)
        curs.execute(sql, args)
        rows = curs.fetchall()
        output = []
        for r in rows:
            filepak = structs.FilePak()
            print(r)
            filepak.file_pak_id = r[0]
            filepak.fullpath = r[1]
            filepak.filename = r[2]
            filepak.md5sum = r[3]
            filepak.record_created_at = r[7]
            filepak.record_updated_at = r[8]
            output.append(filepak)
        return output
