import configparser
import re


class UnrealIniFile:
    def __init__(self, filename=None):
        self._config = None
        self.filename = filename

    @property
    def filename(self):
        return self._filename

    @filename.setter
    def filename(self, val):
        self._filename = val
        if val is not None:
            self._config = configparser.RawConfigParser(strict=False)
            self._config.optionxform = str  # Trick to preserve case in key names
            self._config.read(self._filename)


class GameIniSpecial:
    def __init__(self, filename):
        self._redirect_lines = []
        self._filename = None
        self.filename = filename

    @property
    def filename(self):
        return self._filename

    @filename.setter
    def filename(self, val):
        self._filename = val

    def clear_redirect_references(self):
        self._redirect_lines = []

    def add_redirect_reference(self, pkg_basename, redirect_url, redirect_protocol, relative_path, md5sum):
        args = {
            "pkg_basename": pkg_basename,
            "redirect_protocol": redirect_protocol,
            "redirect_url": redirect_url,
            "relative_path": relative_path,
            "md5sum": md5sum,
        }
        ########### START multi-line awkward indent
        line = '\
RedirectReferences=(PackageName="{pkg_basename}",\
PackageURLProtocol="{redirect_protocol}",\
PackageURL="{redirect_url}/{relative_path}",\
PackageChecksum="{md5sum}")'.format(
            **args
        )
        ########### END multi-line awkward indent

        return self.add_redirect_reference_line(line)

    def add_redirect_reference_line(self, line):
        self._redirect_lines.append(line)
        return line

    def write(self, fp):
        newcontents = None

        with open(self.filename, "r") as inifile:
            curcontents = inifile.read()
            lines_str = "\n".join(self._redirect_lines)

            newcontents = re.sub("RedirectReferences = :PARAM:", lines_str, curcontents)
            has_data = True

        if has_data:
            with open(self.filename, "w") as fp:
                fp.write(newcontents)
